<?php

namespace Drp\EventPalAdmin\Http\Controllers;

use Carbon\Carbon;
use Drp\EventPalAdmin\Http\Requests\NotificationRequest;
use Drp\NativeEventApp\Enums\NotificationStatus;
use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Models\Segment;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Arr;
use Inertia\Controller;
use Inertia\Inertia;
use Inertia\Response;

class NotificationsController extends Controller
{
    public function index(): Response
    {
        return Inertia::render('Notifications/Index', [
            'notifications' => Notification::get()
                ->transform(static function (Notification $notification) {
                    return [
                        'id' => $notification->id,
                        'eventTitle' => Event::findOrFail($notification->event_id)->title,
                        'status' => ucwords(strtolower($notification->status)),
                        'title' => $notification->title,
                        'message' => $notification->message,
                        'sendAt' => optional($notification->send_at)->format('d/m/Y H:i'),
                        'sent' => $notification->sent_at !== null,
                        'totalSent' => $notification->total_sent,
                    ];
                }),
        ]);
    }

    public function create(): Response
    {
        return Inertia::render('Notifications/Create', [
            'events' => Event::all()->toArray(),
            'allSegments' => Segment::all()->toArray(),
            'statuses' => collect(NotificationStatus::getValues())
                ->transform(static function (string $status) {
                    return [
                        'label' => ucwords(strtolower($status)),
                        'value' => $status,
                    ];
                }),
        ]);
    }

    /**
     * Store a new notification.
     */
    public function store(NotificationRequest $request): RedirectResponse
    {
        $data = $request->all();

        $notification = Notification::make(
            Event::findOrFail(Arr::get($data, 'event')),
            Arr::get($data, 'title'),
            Arr::get($data, 'message')
        );

        $notification = $this->updateNotificationSegments($notification, $request);

        if ($request->get('sendNow') === 1) {
            $notification->sendNow();

            return redirect()->route('eventpal.notifications.index')->with('success', 'Notification sent.');
        }

        if ($request->get('sendAt') === ''
            && strtoupper($request->get('status')) === NotificationStatus::DRAFT
        ) {
            $notification->unpublish();

            return redirect()->route('eventpal.notifications.index')
                ->with('success', 'Draft notification created.');
        }

        $notification->sendAt(
            Carbon::parse($request->get('sendAt'))
        );

        if (strtoupper($request->get('status')) === NotificationStatus::PUBLISHED) {
            $notification->publish();

            return redirect()->route('eventpal.notifications.index')->with('success', 'Notification scheduled.');
        }

        $notification->unpublish();

        return redirect()->route('eventpal.notifications.index')
            ->with('success', 'Notification scheduled but is not published.');
    }

    public function edit(Notification $notification): Response
    {
        return Inertia::render('Notifications/Edit', [
            'notification' => [
                'id' => $notification->id,
                'event' => $notification->event_id,
                'status' => $notification->status,
                'title' => $notification->title,
                'message' => $notification->message,
                'sendAt' => optional($notification->send_at)->format('d-m-Y H:i'),
                'sentAt' => $notification->sent_at,
                'segments' => $notification->segments()->pluck('segment_id')->toArray(),
            ],
            'events' => Event::all()->toArray(),
            'allSegments' => Segment::all()->toArray(),
            'statuses' => collect(NotificationStatus::getValues())
                    ->transform(static function (string $status) {
                        return [
                            'label' => ucwords(strtolower($status)),
                            'value' => $status,
                        ];
                    }),
        ]);
    }

    /**
     * Update a notification.
     */
    public function update(Notification $notification, NotificationRequest $request): RedirectResponse
    {
        $data = $request->all();

        $notification->update([
            'event_id' => Event::findOrFail(Arr::get($data, 'event'))->id,
            'title' => Arr::get($data, 'title'),
            'message' => Arr::get($data, 'message'),
            'send_at' => null,
            'sent_at' => null,
        ]);

        $notification = $this->updateNotificationSegments($notification, $request);

        if ($request->get('sendNow') === 1) {
            $notification->sendNow();

            return redirect()->route('eventpal.notifications.index')
                ->with('success', 'Notification updated and sent.');
        }

        if ($request->get('sendAt') === ''
            && strtoupper($request->get('status')) === NotificationStatus::DRAFT
        ) {
            $notification->unpublish();

            return redirect()->route('eventpal.notifications.index')
                ->with('success', 'Notification updated.');
        }

        $notification->sendAt(
            Carbon::parse($request->get('sendAt'))
        );

        if (strtoupper($request->get('status')) === NotificationStatus::PUBLISHED) {
            $notification->publish();

            return redirect()->route('eventpal.notifications.index')
                ->with('success', 'Notification updated and scheduled.');
        }

        $notification->unpublish();

        return redirect()->route('eventpal.notifications.index')
            ->with('success', 'Notification scheduled but is not published.');
    }

    public function destroy(Notification $notification): RedirectResponse
    {
        $notification->delete();

        return redirect()->route('eventpal.notifications.index')
            ->with('success', 'Notification successfully deleted');
    }

    /**
     * Update a notifications segments or detach all if the notification exists and none have been specified.
     */
    private function updateNotificationSegments(Notification $notification, NotificationRequest $request): Notification
    {
        if ($request->filled('segments') && count($request->get('segments')) > 0) {
            return $notification->forSegments(
                collect($request->get('segments'))->map(static function (int $segmentId) {
                    return Segment::findOrFail($segmentId);
                })
            );
        }

        if ($notification->exists) {
            return tap($notification, static function (Notification $notification): void {
                $notification->segments()->detach();
            });
        }

        return $notification;
    }
}
